#include <stdio.h>
#include <math.h>

#define N (1024*1024+3)
#define THREAD_PER_BLOCK 512

__global__ void add( int *a, int *b, int *c , int Nmax ) {
       int index=threadIdx.x+blockIdx.x*blockDim.x;
       if (index < Nmax) 
       {
		c[index] = a[index]+b[index];
       }
}


int main( void ) {
    int *a, *b, *c;               // host copies of a, b, c
    int *dev_a, *dev_b, *dev_c;   // device copies of a, b, c
    int size = N * sizeof( int ); // we need space for N   									// integers
    int i;

    // allocate device copies of a, b, c
    cudaMalloc( (void**)&dev_a, size );
    cudaMalloc( (void**)&dev_b, size );
    cudaMalloc( (void**)&dev_c, size );

    a = (int*)malloc( size ); 
    b = (int*)malloc( size );
    c = (int*)malloc( size );

    for(i=0; i<N; i++) {
		a[i]=i;
		b[i]=-i;
   }

    // copy inputs to device
   cudaMemcpy( dev_a, a, size, cudaMemcpyHostToDevice );
   cudaMemcpy( dev_b, b, size, cudaMemcpyHostToDevice );

    // launch an add() kernel with N threads
    //add<<< N/THREAD_PER_BLOCK, THREAD_PER_BLOCK >>>( dev_a, dev_b, dev_c, N );
    add<<< (N+THREAD_PER_BLOCK-1)/THREAD_PER_BLOCK, THREAD_PER_BLOCK >>>( dev_a, dev_b, dev_c, N );

    // copy device result back to host copy of c
   cudaMemcpy( c, dev_c, size,   cudaMemcpyDeviceToHost );

    for(i=0; i<N; i++) {
	if(c[i] !=0) {
	   printf("error: expected 0, got %d!\n",c[i]);
	   break;
	}
    }
    if(i==N) {
	   printf("N=%d correct \n",N);
    }    

 
    free( a ); free( b ); free( c );
    cudaFree( dev_a );
    cudaFree( dev_b );
    cudaFree( dev_c );
    return 0;
}
